import { DatePipe, CommonModule } from '@angular/common';
import { Injectable, EventEmitter, Pipe, InjectionToken, Inject, NgModule } from '@angular/core';
import { HttpParams, HttpResponse, HttpClient } from '@angular/common/http';
import { of, Subject, throwError, Observable, from } from 'rxjs';
import { map, throttleTime, debounceTime, takeUntil, catchError, mergeMap, tap } from 'rxjs/operators';
import { FormArray, FormGroup } from '@angular/forms';
import { __awaiter } from 'tslib';
import { UserManager } from 'oidc-client';
import * as jwtDecode_ from 'jwt-decode';
import { getTestBed, TestBed } from '@angular/core/testing';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {number} */
const EOData = {
    Equals: 0,
    NotEquals: 1,
    GreaterThan: 2,
    GreaterThanOrEquals: 3,
    LessThan: 4,
    LessThanOrEquals: 5,
    Flag: 6,
    Contains: 7,
    EndsWith: 8,
    StartsWith: 9,
    Length: 10,
    IndexOf: 11,
    SubString: 12,
    ToLower: 13,
    ToUpper: 14,
    Trim: 15,
    ConCat: 16,
    Year: 17,
    Month: 18,
    Day: 19,
    Hour: 20,
    Minute: 21,
    Second: 22,
    Date: 23,
    Time: 24,
    TotalOffSetMinutes: 25,
    Now: 26,
    MinDateTime: 27,
    MaxDateTime: 28,
    Round: 29,
    Floor: 30,
    Ceiling: 31,
    Cast: 32,
    IsOf: 33,
    And: 34,
    Or: 35,
    Not: 36,
    Null: 37,
};
EOData[EOData.Equals] = 'Equals';
EOData[EOData.NotEquals] = 'NotEquals';
EOData[EOData.GreaterThan] = 'GreaterThan';
EOData[EOData.GreaterThanOrEquals] = 'GreaterThanOrEquals';
EOData[EOData.LessThan] = 'LessThan';
EOData[EOData.LessThanOrEquals] = 'LessThanOrEquals';
EOData[EOData.Flag] = 'Flag';
EOData[EOData.Contains] = 'Contains';
EOData[EOData.EndsWith] = 'EndsWith';
EOData[EOData.StartsWith] = 'StartsWith';
EOData[EOData.Length] = 'Length';
EOData[EOData.IndexOf] = 'IndexOf';
EOData[EOData.SubString] = 'SubString';
EOData[EOData.ToLower] = 'ToLower';
EOData[EOData.ToUpper] = 'ToUpper';
EOData[EOData.Trim] = 'Trim';
EOData[EOData.ConCat] = 'ConCat';
EOData[EOData.Year] = 'Year';
EOData[EOData.Month] = 'Month';
EOData[EOData.Day] = 'Day';
EOData[EOData.Hour] = 'Hour';
EOData[EOData.Minute] = 'Minute';
EOData[EOData.Second] = 'Second';
EOData[EOData.Date] = 'Date';
EOData[EOData.Time] = 'Time';
EOData[EOData.TotalOffSetMinutes] = 'TotalOffSetMinutes';
EOData[EOData.Now] = 'Now';
EOData[EOData.MinDateTime] = 'MinDateTime';
EOData[EOData.MaxDateTime] = 'MaxDateTime';
EOData[EOData.Round] = 'Round';
EOData[EOData.Floor] = 'Floor';
EOData[EOData.Ceiling] = 'Ceiling';
EOData[EOData.Cast] = 'Cast';
EOData[EOData.IsOf] = 'IsOf';
EOData[EOData.And] = 'And';
EOData[EOData.Or] = 'Or';
EOData[EOData.Not] = 'Not';
EOData[EOData.Null] = 'Null';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe que representa o operador odata or
 * Deve ser apenas instanciado pelo Odata.Or()
 */
class ODataOr {
    /**
     * Construtor
     * @param {?} left
     * @param {?} right
     */
    constructor(left, right) {
        this.left = left;
        this.right = right;
        /**
         * Operador lógico ||
         */
        this.operator = EOData.Or;
    }
    /**
     * Retorna a string Odata
     * @return {?}
     */
    toString() {
        return '(' + this.left.toString() + ' or ' + this.right.toString() + ')';
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe que representa o operador odata not
 * Deve ser apenas instanciado pelo Odata.not()
 */
class ODataNot {
    /**
     * Construtor
     * @param {?} odataExpression
     */
    constructor(odataExpression) {
        this.odataExpression = odataExpression;
        /**
         * Operador lógico !
         */
        this.operator = EOData.Not;
    }
    /**
     * Retorna a string Odata
     * @return {?}
     */
    toString() {
        return 'not (' + this.odataExpression.toString() + ')';
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe que representa o operador odata and
 * Deve ser apenas instanciado pelo Odata.And()
 */
class ODataAnd {
    /**
     * Construtor
     * @param {?} left
     * @param {?} right
     */
    constructor(left, right) {
        this.left = left;
        this.right = right;
        /**
         * Operador lógico &&
         */
        this.operator = EOData.And;
    }
    /**
     * Retorna a string Odata
     * @return {?}
     */
    toString() {
        return '(' + this.left.toString() + ' and ' + this.right.toString() + ')';
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class QueryFilter {
    constructor() {
        /**
         * query: array que contém os dados de filtro
         */
        this.query = [];
    }
    /**
     * @param {...?} o
     * @return {?}
     */
    static Build(...o) {
        /** @type {?} */
        const filter = new QueryFilter();
        filter.add(...o);
        return filter;
    }
    /**
     * método que retorna o tamanho do array/campo query
     * @return {?}
     */
    get length() {
        return this.query.length;
    }
    /**
     * método que adiciona na query os objetos com informações de filtro
     * @param {...?} o array com os dados de filtro
     * @return {?}
     */
    add(...o) {
        o.forEach((/**
         * @param {?} element
         * @return {?}
         */
        (element) => {
            this.query.push(element);
        }));
    }
    /**
     * método que adiciona um agrupador de lógica ( parenteses ) na última
     * expressão ou na próxima que for adicionada
     * @param {?=} previous
     * @return {?}
     */
    addGroup(previous = true) {
    }
    /**
     * Implementar de forma a compor o filtro com os dados recebidos
     * @param {?} params
     * @return {?}
     */
    updateParams(params) {
        this.query.forEach((/**
         * @param {?} e
         * @return {?}
         */
        (e) => params = params.append(e.field, e.value.toString())));
        return params;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe que representa um chave/valor
 */
class FieldValue {
    /**
     * @param {?} field
     * @param {?} value
     */
    constructor(field, value) {
        this.field = field;
        this.value = value;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {number} */
const ApiOrderType = {
    OrderAsc: 0,
    OrderDesc: 1,
};
ApiOrderType[ApiOrderType.OrderAsc] = 'OrderAsc';
ApiOrderType[ApiOrderType.OrderDesc] = 'OrderDesc';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe específica para tratar erros críticos do sistema
 */
class CriticalError extends Error {
    /**
     * Método construtor que inicializa a classe
     * @param {?} internalError
     * @param {?=} objectData
     */
    constructor(internalError, objectData) {
        super(internalError.toString());
        console.error(internalError);
        if (objectData) {
            console.log(JSON.stringify(objectData));
        }
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Representação do campo, contendo seu nome e o tipo de ordenação
 */
class ApiOrderField {
    /**
     * Retorna a string formatada do campo, já contendo o sinal apropriado a seu tipo de ordenação
     * @return {?}
     */
    toString() {
        /** @type {?} */
        const order = this.orderType === ApiOrderType.OrderAsc ? '+' : '-';
        return `${order}${this.field}`;
    }
    /**
     * Seta a ordem do campo como ascendente
     * @return {?}
     */
    Asc() {
        this.orderType = ApiOrderType.OrderAsc;
        return this;
    }
    /**
     * Seta a ordem do campo como descendente
     * @return {?}
     */
    Desc() {
        this.orderType = ApiOrderType.OrderDesc;
        return this;
    }
    /**
     * Método contrutor da classe
     * @param {?} field
     */
    constructor(field) {
        if (!field || field.length === 0) {
            throw new CriticalError(`Atributo field não pode estar vazio.`, this);
        }
        this.field = field;
        this.orderType = ApiOrderType.OrderAsc;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe com as constantes definidas de data e hora
 */
class ApiDateTimeConstants {
}
/**
 * formato de data padrão
 */
ApiDateTimeConstants.DATE_FORMAT = 'yyyy-MM-dd';
/**
 * formato de exibição de data em português
 */
ApiDateTimeConstants.DATE_FORMAT_DISPLAY_PT = 'dd/MM/yyyy';
/**
 * formato de exibição de data em inglês
 */
ApiDateTimeConstants.DATE_FORMAT_DISPLAY_EN = 'MM/dd/yyyy';
/**
 * formato de exibição padrão de hora, minuto e segundo
 */
ApiDateTimeConstants.TIME_FORMAT = 'HH:mm:ss';
/**
 * formato de exibição padrão de fuso horário
 */
ApiDateTimeConstants.TIMEZONE_FORMAT = 'ZZZZZ';
/**
 * Linguagem padrão
 */
ApiDateTimeConstants.LANG_DEFAULT = 'en-US';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe do serviço de formatação de data e hora de acordo com o guia de implementação de API
 */
class ApiDateTimeService {
    /**
     * Método construtor da classe
     * @param {?} datePipe
     */
    constructor(datePipe) {
        this.datePipe = datePipe;
    }
    /**
     * Retorna data no formato especificado pelo Guia de Implementação de API
     * Formato: data (E8601DAw.): yyyy-mm-dd
     * @param {?} value
     * @return {?}
     */
    dateFormatAPI(value) {
        return this.datePipe.transform(value, ApiDateTimeConstants.DATE_FORMAT);
    }
    /**
     * Retorna data no formato extendido especificado pelo Guia de Implementação de API
     *  Formato: data UTC estendido (E8601DZw.d): yyyy-mm-ddThh:mm:ss.nnnnnn+|-hh:mm   (+|-hh:mm = timezone/fuso)
     * @param {?} value
     * @return {?}
     */
    dateFormatExtendAPI(value) {
        /** @type {?} */
        const result = this.dateFormatAPI(value) + 'T' +
            this.datePipe.transform(value, ApiDateTimeConstants.TIME_FORMAT) +
            this.datePipe.transform(value, ApiDateTimeConstants.TIMEZONE_FORMAT);
        return result;
    }
    /**
     * Retorna hora no formato especificado pelo Guia de Implementação de API
     * Hora (E8601TMw.d): hh:mm:ss.ffffff   (f = milisegundos)
     * @param {?} value
     * @return {?}
     */
    timeFormatAPI(value) {
        /** @type {?} */
        const dateMilliseconds = new Date(value)
            .getMilliseconds()
            .toPrecision(6)
            .replace('.', '');
        /** Retorna no formato do guia de implementação de API para hora:  hh:mm:ss.ffffff */
        return `${this.datePipe.transform(value, ApiDateTimeConstants.TIME_FORMAT)}.${dateMilliseconds} `;
    }
}
ApiDateTimeService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
ApiDateTimeService.ctorParameters = () => [
    { type: DatePipe }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe que implementa as operações de filtro
 */
class OData {
    /**
     * Fábrica estática que retorna um odata do tipo Null
     * @return {?}
     */
    static Null() {
        return new OData(null);
    }
    /**
     * Fábrica estática que retorna um odata do tipo Equals
     * @param {?} field
     * @param {?} value
     * @return {?}
     */
    static Equals(field, value) {
        return new OData({
            field,
            value,
            operator: EOData.Equals
        });
    }
    /**
     * Fábrica estática que retorna um odata do tipo NotEquals
     * @param {?} field
     * @param {?} value
     * @return {?}
     */
    static NotEquals(field, value) {
        return new OData({
            field,
            value,
            operator: EOData.NotEquals
        });
    }
    /**
     * Fábrica estática que retorna um odata do tipo GreaterThan
     * @param {?} field
     * @param {?} value
     * @return {?}
     */
    static GreaterThan(field, value) {
        return new OData({
            field,
            value,
            operator: EOData.GreaterThan
        });
    }
    /**
     * Fábrica estática que retorna um odata do tipo GreaterThanOrEquals
     * @param {?} field
     * @param {?} value
     * @return {?}
     */
    static GreaterThanOrEquals(field, value) {
        return new OData({
            field,
            value,
            operator: EOData.GreaterThanOrEquals
        });
    }
    /**
     * Fábrica estática que retorna um odata do tipo LessThan
     * @param {?} field
     * @param {?} value
     * @return {?}
     */
    static LessThan(field, value) {
        return new OData({
            field,
            value,
            operator: EOData.LessThan
        });
    }
    /**
     * Fábrica estática que retorna um odata do tipo LessThanOrEquals
     * @param {?} field
     * @param {?} value
     * @return {?}
     */
    static LessThanOrEquals(field, value) {
        return new OData({
            field,
            value,
            operator: EOData.LessThanOrEquals
        });
    }
    /**
     * Fábrica estática que retorna um odata do tipo Contains
     * @param {?} field
     * @param {?} value
     * @return {?}
     */
    static Contains(field, value) {
        return new OData({
            field,
            value,
            operator: EOData.Contains
        });
    }
    /**
     * Fábrica estática que retorna um odata para o operador lógico E
     * @param {?} left
     * @param {?} right
     * @return {?}
     */
    static And(left, right) {
        return new ODataAnd(left, right);
    }
    /**
     * Fábrica estática que retorna um odata para o operador lógico Ou
     * @param {?} left
     * @param {?} right
     * @return {?}
     */
    static Or(left, right) {
        return new ODataOr(left, right);
    }
    /**
     * Fábrica estática que retorna um odata para o operador lógico Not
     * @param {?} oDataExpression
     * @return {?}
     */
    static Not(oDataExpression) {
        return new ODataNot(oDataExpression);
    }
    /**
     * Método construtor que inicializa o objeto
     * @private
     * @param {?=} data
     */
    constructor(data = { field: '', value: '', operator: EOData.Equals }) {
        if (data == null) {
            this.operator = EOData.Null;
        }
        else {
            if ((!data.field || data.field.length === 0) && data.operator !== EOData.And && data.operator !== EOData.Or) {
                throw new CriticalError(`Atributo field não pode estar vazio.`, data);
            }
            this.field = data.field;
            this.value = data.value;
            this.operator = data.operator;
        }
    }
    /**
     * método que trata o filtro solicitado pelo usuário, passando para o
     * formato string
     * @return {?}
     */
    toString() {
        /** @type {?} */
        let filter;
        /** @type {?} */
        let value = this.value !== undefined ? this.value : 'null';
        if (typeof value === 'string') {
            if (value !== 'null') {
                value = this.filterPaser(value);
                value = `'${value}'`;
            }
        }
        else {
            try {
                value = this.value.toString();
            }
            catch (e) {
                throw new CriticalError('Valor informado é inválido.', (/** @type {?} */ (value)));
            }
        }
        switch (+this.operator) {
            case EOData.Equals:
                filter = `${this.field} eq ${value}`;
                break;
            case EOData.NotEquals:
                filter = `${this.field} ne ${value}`;
                break;
            case EOData.GreaterThan:
                filter = `${this.field} gt ${value}`;
                break;
            case EOData.GreaterThanOrEquals:
                filter = `${this.field} ge ${value}`;
                break;
            case EOData.LessThan:
                filter = `${this.field} lt ${value}`;
                break;
            case EOData.LessThanOrEquals:
                filter = `${this.field} le ${value}`;
                break;
            case EOData.Contains:
                filter = `contains(${this.field},${value})`;
                break;
            case EOData.Null:
                filter = '';
                break;
            case EOData.And:
            case EOData.Or:
            case EOData.Not:
            case EOData.EndsWith:
            case EOData.StartsWith:
            case EOData.Length:
            case EOData.IndexOf:
            case EOData.SubString:
            case EOData.ToLower:
            case EOData.ToUpper:
            case EOData.Trim:
            case EOData.ConCat:
            case EOData.Year:
            case EOData.Month:
            case EOData.Day:
            case EOData.Hour:
            case EOData.Minute:
            case EOData.Second:
            case EOData.Date:
            case EOData.Time:
            case EOData.TotalOffSetMinutes:
            case EOData.Now:
            case EOData.MinDateTime:
            case EOData.MaxDateTime:
            case EOData.Round:
            case EOData.Floor:
            case EOData.Ceiling:
            case EOData.Cast:
            case EOData.IsOf:
            default:
                throw new CriticalError(`Flag não implementada.`, this);
        }
        return filter;
    }
    /**
     * Função que faz o parser dos parâmetros de busca informados retirando
     * os caracteres especiais
     * @param {?} filter filtro informado
     * @return {?}
     */
    filterPaser(filter) {
        filter = filter.replace(new RegExp('[ÁÀÂÃ]', 'gi'), 'a');
        filter = filter.replace(new RegExp('[ÉÈÊ]', 'gi'), 'e');
        filter = filter.replace(new RegExp('[ÍÌÎ]', 'gi'), 'i');
        filter = filter.replace(new RegExp('[ÓÒÔÕ]', 'gi'), 'o');
        filter = filter.replace(new RegExp('[ÚÙÛ]', 'gi'), 'u');
        filter = filter.replace(new RegExp('[Ç]', 'gi'), 'c');
        return filter.toUpperCase();
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe que recebe os parâmetros solicitados para filtragem
 */
class ApiFilter {
    constructor() {
        /**
         * query: array que contém os dados de filtro
         */
        this.query = [];
    }
    /**
     * método que retorna o tamanho do array/campo query
     * @return {?}
     */
    get length() {
        return this.query.length;
    }
    /**
     * método que adiciona na query os objetos com informações de filtro
     * @param {...?} o array com os dados de filtro
     * @return {?}
     */
    add(...o) {
        /** @type {?} */
        let content = '';
        if (this.next === true) {
            content += '(';
        }
        for (let element of o) {
            if (element === undefined || element === null) {
                element = OData.Null();
            }
            if (typeof element === 'string') {
                content += element;
            }
            else {
                content += element.toString();
            }
            content += ' ';
        }
        content = content.trim();
        if (this.next === true) {
            content += ')';
        }
        this.query.push(content);
        this.next = false;
    }
    /**
     * método que adiciona um agrupador de lógica ( parenteses ) na última
     * expressão ou na próxima que for adicionada
     * @param {?=} previous
     * @return {?}
     */
    addGroup(previous = true) {
        if (previous === true) {
            this.query[this.query.length - 1] = `(${this.query[this.query.length - 1]})`;
        }
        else {
            this.next = true;
        }
    }
    /**
     * método que retorna o objeto no formato string
     * @return {?}
     */
    toString() {
        return this.query.join(' ');
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Esta classe define os métodos e atributos para consulta de dados
 */
class ApiQueryString {
    /**
     * Método construtor que inicializa o objeto
     * @param {?=} query
     */
    constructor(query = {
        page: 1,
        pageSize: 20,
        filter: undefined,
        expand: undefined,
        order: undefined,
        fields: undefined,
        query: undefined
    }) {
        this.page = query.page;
        this.pageSize = query.pageSize;
        this.filter = query.filter;
        this.expand = query.expand;
        this.fields = query.fields;
        this.order = query.order;
        this.query = query.query;
    }
    /**
     * Retorna os parâmetros de consulta(page, pageSize, filter, expand e fields) em formato string
     * @return {?}
     */
    getQueryString() {
        /** @type {?} */
        let pars = new HttpParams();
        if (this.page && this.page.toString()) {
            pars = pars.set('page', this.page.toString());
        }
        if (this.pageSize && this.pageSize.toString()) {
            pars = pars.set('pageSize', this.pageSize.toString());
        }
        if (this.filter && this.filter.toString()) {
            pars = pars.set('filter', this.filter.toString());
        }
        if (this.expand && this.expand.length > 0) {
            pars = pars.set('expand', this.expand.join(','));
        }
        if (this.order && this.order.toString()) {
            pars = pars.set('order', this.order.join(','));
        }
        if (this.fields && this.fields.length > 0) {
            pars = pars.set('fields', this.fields.join(','));
        }
        if (this.query && this.query.length > 0) {
            pars = this.query.updateParams(pars);
        }
        return pars;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe utilitária responsável por controlar a consulta de todos os registros de uma entidade
 * @template Entity, Dto
 */
class MappedGetAll {
    /**
     * Método construtor que inicializa a classe utilitária
     * @param {?} httpClient Injeção do HttpClient
     * @param {?} _urlGetAll URL completa do endpoint de GetAll
     * @param {?} mapper Classe que determina o mapeamento da entidade
     */
    constructor(httpClient, _urlGetAll, mapper) {
        this.httpClient = httpClient;
        this._urlGetAll = _urlGetAll;
        this.mapper = mapper;
        /**
         * Utilitário para manipular parâmetros query na requisição
         */
        this.apiQueryString = new ApiQueryString();
        /**
         * Array com os registros
         */
        this._items = [];
        /**
         * Indica se há mais registros
         */
        this._hasNext = false;
    }
    /**
     * URL completa do endpoint de GetAll
     * @return {?}
     */
    get urlGetAll() {
        return this._urlGetAll;
    }
    /**
     * @return {?}
     */
    get items() {
        return this._items;
    }
    /**
     * Página corrente
     * @return {?}
     */
    get page() {
        return this.apiQueryString.page;
    }
    /**
     * Número de registros por página
     * @param {?} _pageSize
     * @return {?}
     */
    set pageSize(_pageSize) {
        this.apiQueryString.pageSize = _pageSize;
    }
    /**
     * @return {?}
     */
    get pageSize() {
        return this.apiQueryString.pageSize;
    }
    /**
     * @return {?}
     */
    get hasNext() {
        return this._hasNext;
    }
    /**
     * Expansões para a consulta
     * @return {?}
     */
    get expands() {
        return this.apiQueryString.expand;
    }
    /**
     * Método responsável por realizar a consulta dos registros
     * @return {?} Retorno assíncrono dos registros buscados
     */
    getAll() {
        return this.httpClient.get(this.urlGetAll, { params: this.apiQueryString.getQueryString() }).pipe(map((/**
         * @param {?} json
         * @return {?}
         */
        (json) => {
            /** @type {?} */
            const result = this.mapper.MapTo(json);
            this._hasNext = result.hasNext;
            this._items = this._items.concat(result.items);
            return this._items;
        })));
    }
    /**
     * Método responsável por 'limpar' as parametrizações da consulta
     * e realizá-la novamente
     * @param {?=} apiQueryString
     * @return {?} Retorno assíncrono dos registros buscados
     */
    reset(apiQueryString) {
        this._hasNext = false;
        if (apiQueryString) {
            this.apiQueryString = apiQueryString;
        }
        this.apiQueryString.page = 1;
        return this.httpClient.get(this.urlGetAll, { params: this.apiQueryString.getQueryString() }).pipe(map((/**
         * @param {?} json
         * @return {?}
         */
        (json) => {
            this._items = [];
            /** @type {?} */
            const data = this.mapper.MapTo(json);
            this._hasNext = data.hasNext;
            this._items = data.items;
            return this._items;
        })));
    }
    /**
     * Método responsável por retornar os registros da próxima página
     * @return {?} Retorno assíncrono dos registros buscados
     */
    more() {
        if (this.hasNext) {
            this.apiQueryString.page += 1;
            return this.getAll();
        }
        else {
            return of([]);
        }
    }
    /**
     * Método responsável por definir os filtros por query que devem ser aplicados a consulta
     * @param {?} queryFilter Classe que contém a definição dos filtros
     * @return {?}
     */
    setQueryFilter(queryFilter) {
        this.apiQueryString.query = queryFilter;
    }
    /**
     * Método responsável por definir as expansões para a consulta
     * @param {...?} expands Lista de expansões
     * @return {?}
     */
    setExpands(...expands) {
        this.apiQueryString.expand = expands;
    }
    /**
     * Método responsável por 'limpar' as parametrizações da consulta e os itens
     * @return {?}
     */
    resetSearch() {
        this._items = [];
        this._hasNext = false;
        this.apiQueryString.page = 1;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Implementação da lista padrão de Dto
 * @template Dto
 */
class ListDto {
    constructor() {
        /**
         * A própria lista de itens
         */
        this.items = new Array();
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Mapper padrão para Dto
 * @template In, Out
 */
class ListDtoMapper {
    /**
     * @param {?} itemMapper
     */
    constructor(itemMapper) {
        this.itemMapper = itemMapper;
    }
    /**
     * Efetua o mapeando da lista de entidades
     * @param {?} content Conteúdo do backend
     * @return {?}
     */
    MapTo(content) {
        /** @type {?} */
        const result = new ListDto();
        result.hasNext = content.hasNext;
        content.items.forEach((/**
         * @param {?} item
         * @return {?}
         */
        (item) => result.items.push(this.itemMapper.MapTo(item))));
        return result;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Representa a resposta padrão em endpoints de criação
 */
class DefaultResponseDto {
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe mapeadeadora de urls padrão
 * Atende o formato no template /entity/{0}/other/{1}
 */
class UrlMapper {
    /**
     * @param {?} templateUrl
     */
    constructor(templateUrl) {
        this.templateUrl = templateUrl;
    }
    /**
     * Implementação do mapeamento
     *
     * @param {?} content contém o array com o conteúdo ordenado da url
     * @return {?} A url mapeada
     */
    MapTo(content) {
        /** @type {?} */
        let modified = this.templateUrl;
        content.forEach((/**
         * @param {?} d
         * @param {?} i
         * @return {?}
         */
        (d, i) => {
            modified = modified.replace(`{${i}}`, d);
        }));
        return modified;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe abstrata para possuir um mapper de url por padrão
 * @abstract
 */
class MappedHttp {
    /**
     * @param {?} urlMapper Um mapper que implementa um mapeamento de parâmetros para url
     */
    constructor(urlMapper) {
        this.urlMapper = urlMapper;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Realiza a operação de Get para o enpoint informado, mapeando a entidade antes de enviar
 *
 * @template In, Out
 */
class MappedHttpGetWithQueryString extends MappedHttp {
    /**
     * Creates an instance of MappedHttpGet.
     * @param {?} endpoint aceita o formato /api/entity/{0}/other/{1}
     * @param {?} mapper o conversor para o endpoint
     * @param {?} httpClient o cliente http
     * @param {?=} urlMapper
     */
    constructor(endpoint, mapper, httpClient, urlMapper = new UrlMapper(endpoint)) {
        super(urlMapper);
        this.mapper = mapper;
        this.httpClient = httpClient;
        this._expands = [];
        this._fields = [];
    }
    /**
     * Expansões para a consulta
     * @return {?}
     */
    get expands() {
        return this._expands;
    }
    /**
     * Campos para a consulta
     * @return {?}
     */
    get fields() {
        return this._fields;
    }
    /**
     * Método responsável por definir as expansões para a consulta
     * @param {...?} expands Lista de expansões
     * @return {?}
     */
    setExpands(...expands) {
        this._expands = expands;
    }
    /**
     * Método responsável por definir os campos para a consulta
     * @param {...?} fields Lista de campos
     * @return {?}
     */
    setFields(...fields) {
        this._fields = fields;
    }
    /**
     * Implementação da chamada de leitura de um recurso
     *
     * @param {...?} params parâmetros da url
     * @return {?} Deve ser assinado para realizar a leitura do serviço
     */
    Get(...params) {
        /** @type {?} */
        const template = this.urlMapper.MapTo(params);
        return this.httpClient.get(template, { params: this.getQueryString() }).pipe(map((/**
         * @param {?} data
         * @return {?}
         */
        (data) => this.mapper.MapTo(data))));
    }
    /**
     * Retorna os parâmetros de consulta(expand e fields) em formato string
     * @private
     * @return {?}
     */
    getQueryString() {
        /** @type {?} */
        let params = new HttpParams();
        if (this.expands && this.expands.length > 0) {
            params = params.set('expand', this.expands.join(','));
        }
        if (this.fields && this.fields.length > 0) {
            params = params.set('fields', this.fields.join(','));
        }
        return params;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Realiza a operação de Get para o enpoint informado, mapeando a entidade após recebe-la, com opção de filtros na string
 *
 * @template In, Out
 */
class MappedHttpGetWithQueryFilter extends MappedHttp {
    /**
     * Cria uma instância de MappedHttpGetWithQueryFilter.
     * @param {?} endpoint aceita o formato /api/entity/{0}/other/{1}
     * @param {?} mapper o conversor para o endpoint
     * @param {?} httpClient o cliente http
     * @param {?=} urlMapper
     */
    constructor(endpoint, mapper, httpClient, urlMapper = new UrlMapper(endpoint)) {
        super(urlMapper);
        this.mapper = mapper;
        this.httpClient = httpClient;
        this.apiQuery = new ApiQueryString({ page: null, pageSize: null });
    }
    /**
     * Método responsável por definir os filtros por query que devem ser aplicados a consulta
     * @param {?} queryFilter Classe que contém a definição dos filtros
     * @return {?}
     */
    setQueryFilter(queryFilter) {
        this.apiQuery.query = queryFilter;
    }
    /**
     * Implementação da chamada de leitura de um recurso
     *
     * @param {...?} params parâmetros da url
     * @return {?} Deve ser assinado para realizar a leitura do serviço
     */
    Get(...params) {
        /** @type {?} */
        const template = this.urlMapper.MapTo(params);
        return this.httpClient.get(template, { params: this.apiQuery.getQueryString() }).pipe(map((/**
         * @param {?} data
         * @return {?}
         */
        (data) => this.mapper.MapTo(data))));
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Realiza a operação de Post para o enpoint informado, mapeando a entidade antes de enviar
 * Consumindo o ID a partir do header location
 * @template In, Out
 */
class MappedHttpPostLocation extends MappedHttp {
    /**
     * Creates an instance of MappedHttpPost.
     * @param {?} endpoint aceita o formato /api/entity/{0}/other/{1}
     * @param {?} mapper o conversor para o endpoint
     * @param {?} httpClient o cliente http
     * @param {?=} urlMapper
     */
    constructor(endpoint, mapper, httpClient, urlMapper = new UrlMapper(endpoint)) {
        super(urlMapper);
        this.mapper = mapper;
        this.httpClient = httpClient;
    }
    /**
     * Implementação da chamada de criação de um recurso
     *
     * @param {?=} data dados de entrada para o serviço
     * @param {...?} params parâmetros da url
     * @return {?} Deve ser assinado para realizar a criação do recurso e receber o id
     */
    Post(data, ...params) {
        /** @type {?} */
        const template = this.urlMapper.MapTo(params);
        return this.httpClient.post(template, this.mapper.MapTo(data), { observe: 'response' })
            .pipe(map((/**
         * @param {?} res
         * @return {?}
         */
        (res) => res.headers.get('Location').split('/').reverse()[0])));
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Realiza a operação de Put para o enpoint informado, mapeando a entidade antes de enviar
 *
 * @template In, Out, Response
 */
class MappedHttpPut extends MappedHttp {
    /**
     * Creates an instance of MappedHttpPut.
     * @param {?} endpoint aceita o formato /api/entity/{0}/other/{1}
     * @param {?} mapper o conversor para o endpoint
     * @param {?} httpClient o cliente http
     * @param {?=} urlMapper
     */
    constructor(endpoint, mapper, httpClient, urlMapper = new UrlMapper(endpoint)) {
        super(urlMapper);
        this.mapper = mapper;
        this.httpClient = httpClient;
    }
    /**
     * Implementação da chamada de atualização de um recurso
     *
     * @param {?=} data dados de entrada para o serviço
     * @param {...?} params parâmetros da url
     * @return {?} Deve ser assinado para realizar a atualização do recurso
     */
    Put(data, ...params) {
        /** @type {?} */
        const template = this.urlMapper.MapTo(params);
        return this.httpClient.put(template, this.mapper.MapTo(data));
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Realiza a operação de Post para o enpoint informado, mapeando a entidade antes de enviar
 *
 * @template In, Out, Response
 */
class MappedHttpPost extends MappedHttp {
    /**
     * Creates an instance of MappedHttpPost.
     * @param {?} endpoint aceita o formato /api/entity/{0}/other/{1}
     * @param {?} mapper o conversor para o endpoint
     * @param {?} httpClient o cliente http
     * @param {?=} urlMapper
     */
    constructor(endpoint, mapper, httpClient, urlMapper = new UrlMapper(endpoint)) {
        super(urlMapper);
        this.mapper = mapper;
        this.httpClient = httpClient;
    }
    /**
     * Implementação da chamada de criação de um recurso
     *
     * @param {?=} data dados de entrada para o serviço
     * @param {...?} params parâmetros da url
     * @return {?} Deve ser assinado para realizar a criação do recurso
     */
    Post(data, ...params) {
        /** @type {?} */
        const template = this.urlMapper.MapTo(params);
        return this.httpClient.post(template, this.mapper.MapTo(data));
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Realiza a operação de Get para o enpoint informado, mapeando a entidade antes de enviar
 *
 * @template In, Out
 */
class MappedHttpGet extends MappedHttp {
    /**
     * Creates an instance of MappedHttpGet.
     * @param {?} endpoint aceita o formato /api/entity/{0}/other/{1}
     * @param {?} mapper o conversor para o endpoint
     * @param {?} httpClient o cliente http
     * @param {?=} urlMapper
     */
    constructor(endpoint, mapper, httpClient, urlMapper = new UrlMapper(endpoint)) {
        super(urlMapper);
        this.mapper = mapper;
        this.httpClient = httpClient;
    }
    /**
     * Implementação da chamada de leitura de um recurso
     *
     * @param {...?} params parâmetros da url
     * @return {?} Deve ser assinado para realizar a leitura do serviço
     */
    Get(...params) {
        /** @type {?} */
        const template = this.urlMapper.MapTo(params);
        return this.httpClient.get(template).pipe(map((/**
         * @param {?} data
         * @return {?}
         */
        (data) => this.mapper.MapTo(data))));
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Realiza a operação de Delete para o enpoint informado, mapeando a entidade antes de enviar
 * @template EntityId
 */
class MappedHttpDelete extends MappedHttp {
    /**
     * @param {?} endpoint
     * @param {?} httpClient
     * @param {?=} urlMapper
     */
    constructor(endpoint, httpClient, urlMapper = new UrlMapper(endpoint)) {
        super(urlMapper);
        this.httpClient = httpClient;
    }
    /**
     * Implementação da chamada de deleção de um recurso
     *
     * @param {...?} params
     * @return {?} Deve ser assinado para a deleção ocorrer
     */
    Delete(...params) {
        return this.httpClient.delete(this.urlMapper.MapTo(params));
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {number} */
const TotvsMessageType = {
    success: 0,
    warning: 1,
    error: 2,
    information: 3,
};
TotvsMessageType[TotvsMessageType.success] = 'success';
TotvsMessageType[TotvsMessageType.warning] = 'warning';
TotvsMessageType[TotvsMessageType.error] = 'error';
TotvsMessageType[TotvsMessageType.information] = 'information';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Utilitários para formulários
 */
class FormUtils {
    /**
     * @param {?} form
     */
    constructor(form) {
        this.form = form;
    }
    /**
     * Implementação para verificar se o formulário possui algum controle alterado
     * @return {?}
     */
    isFormChanged() {
        /** @type {?} */
        let controls = [];
        if (this.form) {
            controls = controls.concat(this.getControls(this.form));
        }
        while (controls.length > 0) {
            /** @type {?} */
            const control = controls.shift();
            if (control.dirty) {
                return true;
            }
            else if (control instanceof FormArray) {
                /** @type {?} */
                const formArray = (/** @type {?} */ (control));
                controls = controls.concat(formArray.controls);
            }
            else if (control instanceof FormGroup) {
                /** @type {?} */
                const formGroup = (/** @type {?} */ (control));
                controls = controls.concat(this.getControls(formGroup));
            }
        }
        return false;
    }
    /**
     * @private
     * @param {?} form
     * @return {?}
     */
    getControls(form) {
        /** @type {?} */
        const controls = [];
        for (const x in form.controls) {
            if (form.controls[x]) {
                controls.push(form.controls[x]);
            }
        }
        return controls;
    }
    /**
     * Implementação para marcar o formulário como não mechido pelo usuário
     * @return {?}
     */
    markFormPristine() {
        for (const x in this.form.controls) {
            if (this.form.controls[x]) {
                this.form.controls[x].markAsPristine();
            }
        }
    }
    /**
     * Implementação para verificar a validade do formulário como um todo e marcar os campos inválidos
     * @return {?}
     */
    executeFormValidation() {
        /** @type {?} */
        let ret = true;
        /** @type {?} */
        let controls = [];
        if (this.form) {
            controls = controls.concat(this.getControls(this.form));
        }
        while (controls.length > 0) {
            /** @type {?} */
            const control = controls.shift();
            if (control.errors &&
                (!control.errors._isScalar || control.errors.value !== null)) {
                control.markAsDirty({ onlySelf: true });
                ret = false;
            }
            else if (control instanceof FormArray) {
                /** @type {?} */
                const formArray = (/** @type {?} */ (control));
                controls = controls.concat(formArray.controls);
            }
            else if (control instanceof FormGroup) {
                /** @type {?} */
                const formGroup = (/** @type {?} */ (control));
                controls = controls.concat(this.getControls(formGroup));
            }
        }
        return ret;
    }
    /**
     * Retorna o form relacionado;
     * @return {?}
     */
    getForm() {
        return this.form;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Utilitário de formulário para reduzir boilerplate de interações com um serviço que salve dados
 * \@example
 * export class Component implements OnInit, OnDestroy {
 *   private formUtils: FormUtilsWithService;
 *   private formService: IFormService;
 *   ngOnDestroy() {
 *     this.formUtils.destroy();
 *   }
 *   ngOnInit(): void {
 *     this.formUtils = new FormUtilsWithService(this.form, this.formService);
 *   }
 *   //Salvando dados do formulário
 *   salvar() {
 *     this.formUtils.save().subscribe(() => {});
 *   }
 *   // Exemplo exportando comportamento de formulário alterado e salvo
 * \@Output()
 *   get formularioAlterado(): EventEmitter<void> {
 *     return this.formUtils.formChanged;
 *   }
 * \@Output()
 *   get formularioSalvo(): EventEmitter<boolean> {
 *     return this.formUtils.formSaved;
 *   }
 * }
 * @template T
 */
class FormUtilsWithService {
    /**
     * @param {?} form
     * @param {?} service
     * @param {?=} formSaved
     * @param {?=} formChanged
     */
    constructor(form, service, formSaved = new EventEmitter(), formChanged = new EventEmitter()) {
        this.form = form;
        this.service = service;
        this.formSaved = formSaved;
        this.formChanged = formChanged;
        this.destroy$ = new Subject();
        this.formUtils = new FormUtils(form);
        this.form.valueChanges
            .pipe(throttleTime(100), debounceTime(300), takeUntil(this.destroy$))
            .subscribe((/**
         * @return {?}
         */
        () => {
            this.service.data = this.form.value;
            this.formChanged.next(this.service.data);
        }));
    }
    /**
     * Verifica se o formulário foi alterado e está valido.
     * Solicita ao serviço para salvar
     * Emite o resultado
     * @return {?}
     */
    save() {
        /** @type {?} */
        const validForm = this.executeFormValidation();
        /** @type {?} */
        const changedForm = this.isFormChanged();
        if (changedForm && validForm) {
            return this.service.save().pipe(catchError((/**
             * @param {?} error
             * @return {?}
             */
            error => {
                this.formSaved.next(false);
                return throwError(error);
            })), map((/**
             * @param {?} x
             * @return {?}
             */
            x => {
                this.form.setValue(this.service.data);
                this.formSaved.next(true);
                this.formUtils.markFormPristine();
                return x || this.service.data;
            })));
        }
        else if (validForm) {
            this.formSaved.next(true);
        }
        else {
            this.formSaved.next(false);
        }
        return of(this.service.data);
    }
    /**
     * Remove quaisquer subscrições da classe, evitando memory leak.
     * Deve ser chamado no ngOnDestroy do componente
     * @return {?}
     */
    destroy() {
        this.destroy$.next();
        this.destroy$.complete();
    }
    /**
     * Implementação para verificar se o formulário possui algum controle alterado
     * @return {?}
     */
    isFormChanged() {
        return this.formUtils.isFormChanged();
    }
    /**
     * Implementação para marcar o formulário como não mechido pelo usuário
     * @return {?}
     */
    markFormPristine() {
        this.formUtils.markFormPristine();
    }
    /**
     * Implementação para verificar a validade do formulário como um todo e marcar os campos inválidos
     * @return {?}
     */
    executeFormValidation() {
        return this.formUtils.executeFormValidation();
    }
    /**
     * Retorna o form relacionado;
     * @return {?}
     */
    getForm() {
        return this.form;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {number} */
const MessageType = {
    ERROR: 1,
    WARNING: 2,
    SUCESS: 4,
    NOTIFICATION: 8,
    EXTERNAL: 16,
    CRITICAL: 32,
};
MessageType[MessageType.ERROR] = 'ERROR';
MessageType[MessageType.WARNING] = 'WARNING';
MessageType[MessageType.SUCESS] = 'SUCESS';
MessageType[MessageType.NOTIFICATION] = 'NOTIFICATION';
MessageType[MessageType.EXTERNAL] = 'EXTERNAL';
MessageType[MessageType.CRITICAL] = 'CRITICAL';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {number} */
const ShowMessageType = {
    NotificationSuccess: 0,
    NotificationInformation: 1,
    NotificationError: 2,
    NotificationWarning: 3,
    DialogConfirmation: 4,
    DialogAlert: 5,
};
ShowMessageType[ShowMessageType.NotificationSuccess] = 'NotificationSuccess';
ShowMessageType[ShowMessageType.NotificationInformation] = 'NotificationInformation';
ShowMessageType[ShowMessageType.NotificationError] = 'NotificationError';
ShowMessageType[ShowMessageType.NotificationWarning] = 'NotificationWarning';
ShowMessageType[ShowMessageType.DialogConfirmation] = 'DialogConfirmation';
ShowMessageType[ShowMessageType.DialogAlert] = 'DialogAlert';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe utilitarios de mensagem
 */
class MessageUtils {
    /**
     * Retorna o que foi passado formatado.
     * @param {?=} data
     * @return {?}
     */
    static formattable(data) {
        return data && data.length !== 0;
    }
    /**
     * Converte de TotvsMessageType para MessageType
     * @param {?} totvsMessageType
     * @return {?}
     */
    static messageTypeFrom(totvsMessageType) {
        /** @type {?} */
        let type;
        switch (TotvsMessageType[totvsMessageType]) {
            case TotvsMessageType.error:
                type = MessageType.ERROR;
                break;
            case TotvsMessageType.information:
                type = MessageType.NOTIFICATION;
                break;
            case TotvsMessageType.success:
                type = MessageType.SUCESS;
                break;
            case TotvsMessageType.warning:
                type = MessageType.WARNING;
                break;
            default:
                type = MessageType.EXTERNAL;
        }
        return type;
    }
    /**
     * Converte de o type de um totvs message para um showmessagetype, usado no messagedisplayer
     * @param {?} message
     * @return {?}
     */
    static showMessageTypeFromTotvsMessage(message) {
        /** @type {?} */
        let messageType;
        switch (message.type) {
            default:
            case 'error':
                messageType = ShowMessageType.DialogAlert;
                break;
            case 'information':
                messageType = ShowMessageType.NotificationInformation;
                break;
            case 'warning':
                messageType = ShowMessageType.NotificationInformation;
                break;
            case 'success':
                messageType = ShowMessageType.NotificationInformation;
                break;
        }
        return messageType;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe de texto da mensagem
 */
class TextMessage {
    /**
     * Construtor da classe
     * @param {?} msg
     */
    constructor(msg) {
        this.text = msg.text;
        this.detailText = msg.detailText;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe base de internacionalização
 * @export
 * @abstract
 */
class I18nBaseService {
    /**
     * Construtor da classe
     * @param {?} literalLoader
     */
    constructor(literalLoader) {
        this.literalLoader = literalLoader;
        /**
         * Opções de internacionalização
         */
        this.options = { language: '', url: '' };
    }
    /**
     * Método responsável por setar os atributos definidos da internacionalização context, language, url
     * @param {?} options
     * @return {?}
     */
    updateOptions(options) {
        if (options === undefined) {
            throw new CriticalError('options não definido.', this);
        }
        try {
            Object.assign(this.options, options);
            return this.literalLoader.updateOptions(this.options)
                .pipe(map((/**
             * @param {?} res
             * @return {?}
             */
            res => this.literals = res)));
        }
        catch (error) {
            throw new CriticalError(error);
        }
    }
    /**
     * Método responsável por setar o formato do valor
     * @param {?} data
     * @param {?} text
     * @return {?}
     */
    format(data, text) {
        if (MessageUtils.formattable(data)) {
            data.forEach((/**
             * @param {?} d
             * @param {?} i
             * @return {?}
             */
            (d, i) => {
                text = text.replace(`{${i}}`, d);
            }));
            return text;
        }
        else {
            return text;
        }
    }
    /**
     * Método responsável por buscar e setar a literal do detalhe da mensagem
     * @param {?} baseMsg
     * @return {?}
     */
    text(baseMsg) {
        /** @type {?} */
        let msg;
        try {
            /** @type {?} */
            let detail = '';
            if (baseMsg.detail) {
                detail = this.format(baseMsg.detailData, this.literals[baseMsg.detail]);
            }
            msg = new TextMessage({
                text: this.format(baseMsg.literalData, this.literals[baseMsg.literal]),
                detailText: detail
            });
        }
        catch (error) {
            // tslint:disable-next-line:max-line-length
            throw new CriticalError(`Literais não encontradas '${baseMsg.literal}'/'${baseMsg.detail}'`);
        }
        return msg;
    }
    /**
     * Método responsável por retornar se tem literal
     * @param {?} literal
     * @return {?}
     */
    hasLiteral(literal) {
        try {
            return this.literals[literal] !== undefined;
        }
        catch (error) {
            console.log(error);
            return false;
        }
    }
    /**
     * Método responsável por retornar as opções de linguagem
     * @return {?}
     */
    getLanguage() {
        return this.options.language;
    }
    /**
     * Método responsável por retornar os idiomas suportados pela aplicação
     * @return {?}
     */
    getSupportedLanguages() {
        return this.options.supportedLanguages;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe abstrata padrão de loading
 * @abstract
 */
class LoadingDefault {
    /**
     * Execução do loading
     * @template Z
     * @param {?} callback
     * @param {...?} args
     * @return {?}
     */
    executeLoading(callback, ...args) {
        this.showLoading();
        /** @type {?} */
        const fn = (/** @type {?} */ (callback(...args)));
        if (fn instanceof Promise) {
            fn
                .then((/**
             * @param {?} data
             * @return {?}
             */
            (data) => { this.hideLoading(); return data; }))
                .catch((/**
             * @param {?} error
             * @return {?}
             */
            (error) => { this.hideLoading(); return error; }));
        }
        else if (fn instanceof Observable) {
            /** @type {?} */
            const obs = (/** @type {?} */ (fn));
            obs.pipe(map((/**
             * @param {?} data
             * @return {?}
             */
            (data) => { this.hideLoading(); return data; })))
                .subscribe();
        }
        else {
            setTimeout((/**
             * @return {?}
             */
            () => this.hideLoading()), 50);
        }
        return fn;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe abstrata de apresentação de mensagem
 * @abstract
 */
class AbstractShowMessage {
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {number} */
const ShowMessageLinkActionType = {
    No: 0,
    LinkNewTab: 1,
    Method: 2,
};
ShowMessageLinkActionType[ShowMessageLinkActionType.No] = 'No';
ShowMessageLinkActionType[ShowMessageLinkActionType.LinkNewTab] = 'LinkNewTab';
ShowMessageLinkActionType[ShowMessageLinkActionType.Method] = 'Method';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe com as opções de apresentação de mensagem
 */
class ShowMessageOptions {
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe base de mensagens
 */
class BaseMessage {
    /**
     * Método construtor da classe
     * @param {?} msg
     */
    constructor(msg) {
        /**
         * Tipo da mensagem que carrega
         */
        this._messageType = ShowMessageType.NotificationSuccess;
        this.literal = msg.literal;
        if (msg.detail) {
            this.detail = msg.detail;
        }
    }
    /**
     * Tipo da mensagem que carrega
     * @return {?}
     */
    get messageType() {
        return this._messageType;
    }
    /**
     * Builder para setar o tipo de mensagem
     * @param {?} messageType
     * @return {?}
     */
    withMessageType(messageType) {
        this._messageType = messageType;
        return this;
    }
    /**
     * Método que seta a literal
     * @param {?} literal
     * @return {?}
     */
    set literal(literal) {
        try {
            this.isLiteralValid(literal);
        }
        catch (error) {
            throw (error);
        }
        this._literal = literal;
    }
    /**
     * Método que busca a literal
     * @return {?}
     */
    get literal() {
        return this._literal;
    }
    /**
     * Método que seta o detalhe da literal
     * @param {?} detail
     * @return {?}
     */
    set detail(detail) {
        try {
            this.isLiteralValid(detail);
        }
        catch (error) {
            throw (error);
        }
        this._detail = detail;
    }
    /**
     * Método que busca o detalhe da literal
     * @return {?}
     */
    get detail() {
        return this._detail;
    }
    /**
     * Facilita a adição de dados para formatação da literal
     * @template THIS
     * @this {THIS}
     * @param {?} data
     * @return {THIS}
     */
    withLiteralData(data) {
        (/** @type {?} */ (this))._literalData = data;
        return (/** @type {?} */ (this));
    }
    /**
     * Método que busca a literal
     * @return {?}
     */
    get literalData() {
        return this._literalData;
    }
    /**
     * Facilita a adição de dados para formatação do detalhe da literal
     * @template THIS
     * @this {THIS}
     * @param {?} data
     * @return {THIS}
     */
    withDetailData(data) {
        (/** @type {?} */ (this))._detailData = data;
        return (/** @type {?} */ (this));
    }
    /**
     * Método que busca o detalhe da literal
     * @return {?}
     */
    get detailData() {
        return this._detailData;
    }
    /**
     * Método que busca a literal formatada
     * @return {?}
     */
    get literalFormattable() {
        return MessageUtils.formattable(this._literalData);
    }
    /**
     * Método que busca o detalhe da literal formatado
     * @return {?}
     */
    get detailFormattable() {
        return MessageUtils.formattable(this._detailData);
    }
    /**
     * Método de internacionalização de mensagem
     * @param {?} i18n
     * @return {?}
     */
    messageI18n(i18n) {
        return i18n.text(this);
    }
    /**
     * Método que verifica se literal é válida
     * @protected
     * @param {?} literal
     * @return {?}
     */
    isLiteralValid(literal) {
        if (literal === undefined) {
            throw new CriticalError('Literal não pode ser undefined.');
        }
        else if (literal.length === 0) {
            throw new CriticalError('Literal não pode ser uma string vazia.');
        }
        else if (literal.search(' ') !== -1) {
            throw new CriticalError('Literal não pode conter espaços.');
        }
        try {
            Object.create({ literal });
        }
        catch (erro) {
            throw new CriticalError(erro);
        }
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe abstrata de conversor e exibição de Hora
 * @abstract
 */
class TimeViewConverter {
    /**
     * Construtor da classe
     * @param {?} transformer
     */
    constructor(transformer) {
        this.transformer = transformer;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe/pipe para aplicação de transformação em um objeto data
 */
class TimePipe {
    /**
     * Método construtor da classe
     * @param {?} timeViewConverter
     * @param {?} i18n
     */
    constructor(timeViewConverter, i18n) {
        this.timeViewConverter = timeViewConverter;
        this.i18n = i18n;
    }
    /**
     * Método padrão para aplicação do Pipe
     * @param {?} value recebe o objeto que contém as informações
     * @param {?} format  recebe as informações de formato
     * @return {?}
     */
    transform(value, format) {
        this.timeViewConverter.language = this.i18n.getLanguage();
        return this.timeViewConverter.toView(value, format);
    }
}
TimePipe.decorators = [
    { type: Pipe, args: [{
                name: 'time',
                pure: false
            },] }
];
/** @nocollapse */
TimePipe.ctorParameters = () => [
    { type: TimeViewConverter },
    { type: I18nBaseService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe padrão do serviço de conversão e exibição de Hora
 */
class DefaultTimeViewConverterService extends TimeViewConverter {
    /**
     * Contrutor da classe
     * @param {?} transformer
     */
    constructor(transformer) {
        super(transformer);
        this.transformer = transformer;
    }
    /**
     * Método responsável pela visualização da hora formatada
     * @param {?} value
     * @param {?=} format
     * @return {?}
     */
    toView(value, format) {
        /** @type {?} */
        const timeFormat = format ? format : ApiDateTimeConstants.TIME_FORMAT;
        return this.transformer.transform(value, timeFormat);
    }
}
DefaultTimeViewConverterService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
DefaultTimeViewConverterService.ctorParameters = () => [
    { type: DatePipe }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe abstrata de conversor e exibição de data
 * @abstract
 */
class DateViewConverter {
    /**
     * Método construtor da classe
     * @param {?} transformer
     */
    constructor(transformer) {
        this.transformer = transformer;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe padrão do serviço de conversão e exibição de data
 */
class DefaultDateViewConverterService extends DateViewConverter {
    /**
     * Método construtor da classe
     * @param {?} transformer
     */
    constructor(transformer) {
        super(transformer);
        this.transformer = transformer;
    }
    /**
     * Método responsável pela visualização do valor formatado
     * @param {?} value
     * @param {?=} format
     * @return {?}
     */
    toView(value, format) {
        /** @type {?} */
        let formatDate;
        if (format) {
            formatDate = format;
        }
        else {
            if (this.language && !this.language.includes('pt')) {
                formatDate = ApiDateTimeConstants.DATE_FORMAT_DISPLAY_EN;
            }
            else {
                formatDate = ApiDateTimeConstants.DATE_FORMAT_DISPLAY_PT;
            }
        }
        return this.transformer.transform(value, formatDate);
    }
}
DefaultDateViewConverterService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
DefaultDateViewConverterService.ctorParameters = () => [
    { type: DatePipe }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe/pipe para aplicação de transformação em um objeto data
 */
class DateTimePipe {
    /**
     * Método construtor da clase
     * @param {?} dateViewConverter
     * @param {?} i18n
     */
    constructor(dateViewConverter, i18n) {
        this.dateViewConverter = dateViewConverter;
        this.i18n = i18n;
    }
    /**
     * Método padrão para aplicação do Pipe
     * @param {?} value recebe o objeto que contém as informações
     * @param {?} format  recebe as informações de formato
     * @return {?}
     */
    transform(value, format) {
        this.dateViewConverter.language = this.i18n.getLanguage();
        return this.dateViewConverter.toView(value, format);
    }
}
DateTimePipe.decorators = [
    { type: Pipe, args: [{
                name: 'dateTime',
                pure: false
            },] }
];
/** @nocollapse */
DateTimePipe.ctorParameters = () => [
    { type: DateViewConverter },
    { type: I18nBaseService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {number} */
const AuthHandlerType = {
    /** Tipos possíveis de autenticação */
    oauth2: 0,
};
AuthHandlerType[AuthHandlerType.oauth2] = 'oauth2';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class UrlUtils {
    /**
     * Substitui o placeholder {tenant} pelo shortname do tenanat presente na Url atual
     * @return {?}
     */
    static GetTenantFromUrl() {
        return window.location.origin.split('//')[1].split('.')[0];
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const jwtDecode = jwtDecode_;
/**
 * Token de configuração das informações
 * @type {?}
 */
const AUTH_SETTINGS = new InjectionToken('AUTH_SETTINGS: IAuthSettings Configurações de gerenciamento de sessão e token');
/**
 * Configuração do tipo de autenticação
 * @type {?}
 */
const AUTH_TYPE = new InjectionToken('AUTH_TYPE: IAuthHandlerType Token com o tipo de autenticação usada');
/**
 * Classe abstrata do serviço de autorização
 * @abstract
 */
class AuthBaseService {
    /**
     * Método construtor da classe
     * @param {?} authSettings Token de injeção com as configurações de autenticação
     */
    constructor(authSettings) {
        /**
         * @ignore
         */
        this._isInitialized = false;
        /**
         * Evento emitido nas situações em que o usuário está logado
         */
        this.userLoadedEvent = new EventEmitter();
        /**
         * Evento emitido nas situações em que o token de acesso expirou
         */
        this.accessTokenExpired = new EventEmitter();
        /**
         * Evento emitido de acordo com o número de segundos definido para disparar
         * o aviso antes da expiração do token. (Número de segundos definido
         * no atributo accessTokenExpiringNotificationTime do token de injeção AUTH_SETTINGS)
         */
        this.accessTokenExpiring = new EventEmitter();
        this.authSettingsTemplate = authSettings;
        this.authSettingsUsed = authSettings;
        if (!this.authSettingsUsed.initLazy) {
            this.setShortNameTenant(UrlUtils.GetTenantFromUrl());
        }
    }
    /**
     * Nome abrevido do tenant corrente identificado no serviço de autorização
     * @return {?}
     */
    get shortNameTenant() {
        return this._shortNameTenant;
    }
    /**
     * Propriedade que indica que o serviço foi inicializado devido a possibilidade de
     * carregar o serviço de forma lazy
     * @return {?}
     */
    get isInitialized() {
        return this._isInitialized;
    }
    /**
     * Método responsável por inicializar o UserManager
     * @private
     * @param {?} authSettings Configurações de gerenciamento de sessão e token
     * @return {?}
     */
    init(authSettings) {
        return __awaiter(this, void 0, void 0, function* () {
            this.manager = new UserManager(authSettings);
            // Carrega o usuário atualmente autenticado
            /** @type {?} */
            const user = yield this.getUser();
            this._currentUser = user;
            if (this._currentUser) {
                this.userLoadedEvent.emit(this._currentUser);
            }
            this.addEventAccessTokenExpired();
            this.addEventAccessTokenExpiring();
            this._isInitialized = true;
            return user;
        });
    }
    /**
     * Método responsável por identificar o tenant para o serviço de autorização.
     * Ao setar, o serviço é inicializado novamente
     * @param {?} shortNameTenant Nome abreviado do tenant
     * @return {?} Usuário atualmente autenticado
     */
    setShortNameTenant(shortNameTenant) {
        return __awaiter(this, void 0, void 0, function* () {
            this._shortNameTenant = shortNameTenant;
            /** @type {?} */
            const authority = this.authSettingsTemplate.authority.replace(`{tenant}`, this.shortNameTenant);
            /** @type {?} */
            const authSettings = Object.assign({}, this.authSettingsUsed, { acr_values: `tenant:${this.shortNameTenant}` }, { authority });
            this.authSettingsUsed = authSettings;
            return yield this.init(this.authSettingsUsed);
        });
    }
    /**
     * Método responsável por alterar as configurações de autenticação
     * @param {?} authSettings Configurações de autenticação
     * @return {?}
     */
    changeAuthSettings(authSettings) {
        return __awaiter(this, void 0, void 0, function* () {
            this.authSettingsTemplate = authSettings;
            this.authSettingsUsed = authSettings;
        });
    }
    /**
     * Adiciona o evento de aviso que o token irá expirar, de acordo com o número de segundos
     * definido no atributo accessTokenExpiringNotificationTime do token de injeção AUTH_SETTINGS
     * @private
     * @return {?}
     */
    addEventAccessTokenExpiring() {
        this.manager.events.addAccessTokenExpiring((/**
         * @param {?} e
         * @return {?}
         */
        e => {
            this.accessTokenExpiring.emit(true);
        }));
    }
    /**
     * Adiciona o evento de token expirado, após o token expirar o mesmo é disparado
     * @private
     * @return {?}
     */
    addEventAccessTokenExpired() {
        this.manager.events.addAccessTokenExpired((/**
         * @param {?} e
         * @return {?}
         */
        e => {
            this.accessTokenExpired.emit(true);
        }));
    }
    /**
     * Método responsável por retornar um observable indicando se o usuário
     * está logado
     * @return {?}
     */
    isLoggedInObs() {
        return from(this.manager.getUser()).pipe(map((/**
         * @param {?} user
         * @return {?}
         */
        user => {
            if (user && !user.expired) {
                return true;
            }
            else {
                return false;
            }
        })));
    }
    /**
     * Método responsável por retornar o tipo do token
     * @return {?}
     */
    getUserTokenType() {
        return this._currentUser.token_type;
    }
    /**
     * Método responsável por retornar o token de acesso
     * @return {?}
     */
    getUserAccessToken() {
        return this._currentUser.access_token;
    }
    /**
     * Método responsável por retornar o usuário autenticado
     * @return {?}
     */
    getUser() {
        return this.manager.getUser().then((/**
         * @param {?} x
         * @return {?}
         */
        x => {
            if (x && x.access_token) {
                try {
                    /** @type {?} */
                    const decodedJsonToken = jwtDecode(x.access_token);
                    x.profile.roles = decodedJsonToken.role;
                    if (!Array.isArray(x.profile.roles)) {
                        x.profile.roles = [x.profile.roles];
                    }
                    Object.assign(x.profile, this.extractDataFrom(decodedJsonToken));
                }
                catch (error) {
                    console.error(error);
                }
            }
            return x;
        }));
    }
    /**
     * Utiliza o parâmetro extractFromTokenToUserProfile para extrair apenas os dados especificados
     * @param {?} decodedJsonToken O objeto json decodificado
     * @return {?}
     */
    extractDataFrom(decodedJsonToken) {
        /** @type {?} */
        const obj = {};
        if (this.authSettingsUsed.extractFromTokenToUserProfile) {
            this.authSettingsUsed.extractFromTokenToUserProfile.forEach((/**
             * @param {?} mapping
             * @return {?}
             */
            mapping => {
                if (!decodedJsonToken[mapping.from]) {
                    console.error(`Atributo ${mapping.from} não encontrado no token'${JSON.stringify(decodedJsonToken, undefined, 2)}'`);
                    return;
                }
                Object.assign(obj, mapping.to(decodedJsonToken[mapping.from]));
            }));
        }
        return obj;
    }
    /**
     * Método responsável por retornar o header de autorização
     * @return {?}
     */
    getAuthorizationHeaderValue() {
        return this._currentUser
            ? `${this._currentUser.token_type} ${this._currentUser.access_token}`
            : '';
    }
    /**
     * Método responsável por redirecionar a janela atual para o endpoint
     * de autorização
     * @return {?}
     */
    signinRedirect() {
        this.manager.signinRedirect();
    }
    /**
     * Método responsável por redirecionar a janela atual para
     * o endpoint de término da sessão
     * @return {?}
     */
    signoutRedirect() {
        this.manager.signoutRedirect();
    }
    /**
     * Método responsável por processar a resposta do endpoint de autorização,
     * retornando o usuário autenticado
     * @return {?}
     */
    signinRedirectCallback() {
        if (window.location.hash) {
            window.location.hash = decodeURIComponent(window.location.hash);
            return this.manager.signinRedirectCallback();
        }
        else {
            return this.getUser();
        }
    }
    /**
     * Método disparado ao completar autenticação, emitindo evento caso
     * o usuário esteja realmente autenticado, ou redirecionando para o endpoint de autorização
     * caso não esteja autenticado
     * @return {?}
     */
    completeAuthentication() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.signinRedirectCallback();
            this._currentUser = yield this.getUser();
            if (this._currentUser) {
                this.userLoadedEvent.emit(this._currentUser);
            }
        });
    }
    /**
     * Método responsável por autorizar o direcionamento para uma determinada rota,
     * caso o usuário esteja expirado executa o processo de signout, caso esteja inválido
     * executa a função para o processo de login
     * @return {?}
     */
    authorize() {
        return this.getUser().then((/**
         * @param {?} user
         * @return {?}
         */
        user => {
            if (user) {
                if (user.expired) {
                    this.logoutFlow();
                    return false;
                }
                return true;
            }
            this.loginFlow();
            return false;
        }));
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe que adiciona ao cabeçalho da requisição o token de autorização
 */
class AuthInterceptorHandlerService {
    /**
     * Creates an instance of AuthInterceptorHandlerService.
     * @param {?} auth
     * @param {?} authType
     */
    constructor(auth, authType) {
        this.auth = auth;
        this.authType = authType;
    }
    /**
     * Implementação da interface de interceptação HTTP
     * que apenas adiciona ao cabeçalho da requisição o token.
     * @param {?} request
     * @param {?} next
     * @return {?}
     */
    intercept(request, next) {
        if (this.authType.type === AuthHandlerType.oauth2 && this.auth.isInitialized) {
            return from(this.auth.getUser())
                .pipe(mergeMap((/**
             * @param {?} user
             * @return {?}
             */
            user => {
                /** @type {?} */
                const headers = request.headers.set('Authorization', user ? `${user.token_type} ${user.access_token}` : '');
                /** @type {?} */
                const reqClone = request.clone({
                    headers,
                    url: request.url.replace('{tenant}', this.auth.shortNameTenant || '')
                });
                return next.handle(reqClone);
            })));
        }
        else {
            return next.handle(request);
        }
    }
}
AuthInterceptorHandlerService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
AuthInterceptorHandlerService.ctorParameters = () => [
    { type: AuthBaseService },
    { type: undefined, decorators: [{ type: Inject, args: [AUTH_TYPE,] }] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Constante usado nas requisições para que os erros não sejam exibidos pelo interceptor
 * @type {?}
 */
const NO_ERROR_HEADER_PARAM = 'X-Totvs-No-Error';
/**
 * Classe base de interceptadores http.
 * Pode ser extendida ou apenas implementar IHttpInterceptor e usar de seus métodos estáticos.
 * @abstract
 */
class HttpInterceptorBaseService {
    /**
     * Interceptador de respostas http padrão
     * @param {?} request HttpRequest<any>
     * @param {?} next HttpHandler
     * @param {?} instance Instancia que implementa IHttpInterceptor
     * @return {?}
     */
    static defaultInterceptor(request, 
    /** Próxima manipulação http */
    next, 
    /** Instância do interceptador */
    instance) {
        /** @type {?} */
        const cloneRequest = request.clone();
        request = request.headers.has(NO_ERROR_HEADER_PARAM)
            ? HttpInterceptorBaseService.cloneRequestWithoutErrorsHeaders(request)
            : request;
        return next.handle(request).pipe(tap((/**
         * @param {?} response
         * @return {?}
         */
        (response) => {
            if (response instanceof HttpResponse) {
                instance.interceptResponse(response);
            }
        }), (/**
         * @param {?} error
         * @return {?}
         */
        (error) => {
            instance.interceptError(error, cloneRequest);
        })));
    }
    /**
     * Retorna se, para a requisição enviada, as notificações de erro deverão ser desativadas
     * @param {?} request
     * @return {?}
     */
    static isErrorNotificationDisabled(request) {
        /** @type {?} */
        const noErrorParam = request && request.headers.get(NO_ERROR_HEADER_PARAM);
        return noErrorParam && noErrorParam.toString().toLocaleLowerCase() === 'true';
    }
    /**
     * Clona os dados da requisição, removendo o cabeçalho 'X-Totvs-No-Error'
     * @param {?} request
     * @return {?}
     */
    static cloneRequestWithoutErrorsHeaders(request) {
        return request && request.clone({ headers: request.headers.delete(NO_ERROR_HEADER_PARAM) });
    }
    /**
     * Recupera e formata um título vindo de um ApiTotvsErrors
     * @param {?} totvsError
     * @return {?}
     */
    static titleFromTotvsError(totvsError) {
        /** @type {?} */
        let title = totvsError.code ? totvsError.code : '';
        title = title ? title : totvsError.message;
        title = title ? title : 'requestFailed';
        return title + (totvsError.code && totvsError.message && totvsError.detailedMessage ? ' > ' : '');
    }
    /**
     * Recupera e formata o detalhe vindo de um ApiTotvsErrors
     * @param {?} totvsError
     * @return {?}
     */
    static detailFromTotvsError(totvsError) {
        /** @type {?} */
        let detail = totvsError.code ? totvsError.message : totvsError.detailedMessage;
        if (detail === undefined) {
            detail = totvsError.detailedMessage;
        }
        detail = HttpInterceptorBaseService.isCompleteMessage(totvsError)
            ? `mensagem:${detail}\ndetalhe:${totvsError.detailedMessage}`
            : detail;
        if (totvsError.details && totvsError.details instanceof Array) {
            totvsError.details.forEach((/**
             * @param {?} e
             * @return {?}
             */
            (e) => {
                detail = detail.concat('\n', '-'.repeat(20), '\n', HttpInterceptorBaseService.detailFromTotvsError(e));
            }));
        }
        if (detail === undefined) {
            detail = '';
        }
        return detail;
    }
    /**
     * Retorna um objeto Message a partir de um erro totvs
     * @param {?} totvsError
     * @return {?}
     */
    static messageFromTotvsError(totvsError) {
        return new TextMessage({
            text: HttpInterceptorBaseService.titleFromTotvsError(totvsError),
            detailText: HttpInterceptorBaseService.detailFromTotvsError(totvsError)
        });
    }
    /**
     * Indica se ApiTotvsErrors deverá ser tratado como uma mensagem completa
     * @param {?} e
     * @return {?}
     */
    static isCompleteMessage(e) {
        return e.code && e.message && e.detailedMessage;
    }
    /**
     * Intercepta e realiza o tratamento da requisição/retorno
     * @param {?} request
     * @param {?} next
     * @return {?}
     */
    intercept(request, next) {
        return HttpInterceptorBaseService.defaultInterceptor(request, next, this);
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Armazenador estático para usuário corrente
 */
class CurrentUser {
}
CurrentUser.currentUser = null;

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Mapper que implementa o mapeamento que retorna o próprio tipo
 * @template type
 */
class AnyGetDtoMapper {
    /**
     * Método que retorna o conteúdo enviado
     * @param {?} content
     * @return {?}
     */
    MapTo(content) {
        return content;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe que mapeia a entidade que representa os usuários que serão manipulados em tela
 */
class GetAllUsersDtoMapper {
    /**
     * Método que realiza o mapeamento
     * @param {?} content Entidade de usuários recebida do RAC
     * @return {?} DTO de usuários retornados do RAC que serão utilizado na tela base
     */
    MapTo(content) {
        return content;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const USER_PERMISSION_SETTINGS = new InjectionToken('USER_PERMISSION_SETTINGS: IUserPermissionServiceSettings Token do serviço de apoio de usuário');
/**
 * Classe de serviço que gerencia a parte de permissões do usuário
 */
class UserPermissionService {
    /**
     * Método construtor da classe
     * @param {?} receiveSettings
     * @param {?} httpClient Serviço Http
     */
    constructor(receiveSettings, httpClient) {
        this.receiveSettings = receiveSettings;
        this.httpClient = httpClient;
        this.initService(this.receiveSettings);
    }
    /**
     * @param {?} queryFilter
     * @return {?}
     */
    setQueryFilter(queryFilter) {
        this.mappedGetAll.setQueryFilter(queryFilter);
    }
    /**
     * Método que realiza a requisição http para o endpoint de listagem de usuários
     *
     * @param {?=} queryFilter
     * @return {?}
     */
    getAllUsers(queryFilter) {
        this.mappedGetAll.setQueryFilter(queryFilter);
        return this.mappedGetAll.reset();
    }
    /**
     * Método que realiza a requisição http para o endpoint de busca de um usuário
     * @param {?} userId Identificador do usuário
     * @return {?}
     */
    getUser(userId) {
        return this.mappedGetUser.Get(userId);
    }
    /**
     * Método que realiza a requisição http para o endpoint de busca de informações do usuário logado
     * @return {?}
     */
    getUserInfo() {
        return this.mappedGetUserInfo.Get();
    }
    /**
     * Método que permite a alteração das configurações do serviço de permissão de usuários
     * indicando URLs diferentes para os endpoints
     * @param {?} settings Configurações
     * @return {?}
     */
    changeSettings(settings) {
        return __awaiter(this, void 0, void 0, function* () {
            this.initService(settings);
        });
    }
    /**
     * Método que realizar a inicialização do serviço utilizando as configurações recebidas
     * @private
     * @param {?} settings Configurações
     * @return {?}
     */
    initService(settings) {
        this.settings = settings;
        this.mappedGetAll = new MappedGetAll(this.httpClient, this.settings.allUsersEndpoint, new ListDtoMapper(new GetAllUsersDtoMapper()));
        if (this.settings.userEndpoint) {
            this.mappedGetUser = new MappedHttpGet(this.settings.userEndpoint, new AnyGetDtoMapper(), this.httpClient);
        }
        if (this.settings.userInfoEndpoint) {
            this.mappedGetUserInfo = new MappedHttpGet(this.settings.userInfoEndpoint, new AnyGetDtoMapper(), this.httpClient);
        }
    }
}
UserPermissionService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
UserPermissionService.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [USER_PERMISSION_SETTINGS,] }] },
    { type: HttpClient }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe utilitária para permissões do usuário
 */
class UserPermission {
    /**
     * Método estático responsável por adicionar a validação de permissão
     * @param {?} user
     * @return {?}
     */
    static AddPermissionFeatures(user) {
        /** @type {?} */
        const userWithPermissions = (/** @type {?} */ (user));
        userWithPermissions.permissions = [];
        userWithPermissions.hasPermission = this.hasPermission;
        return userWithPermissions;
    }
    /**
     * Método estático responsável por retornar se um usuário possui a permissão solicitada
     * @this {?}
     * @param {?} permission Permissão a ser verificada
     * @return {?}
     */
    static hasPermission(permission) {
        /** @type {?} */
        const permissions = this.permissions || [];
        /** @type {?} */
        const currentPermission = permissions.find((/**
         * @param {?} p
         * @return {?}
         */
        p => p.key.toLowerCase() === permission.toLowerCase()));
        if (currentPermission) {
            return currentPermission.value;
        }
        return false;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe do serviço de exibição de mensagem
 */
class MessageDisplayerService {
    /**
     * Construtor da classe
     * @param {?} i18n
     * @param {?} messageShower
     */
    constructor(i18n, messageShower) {
        this.i18n = i18n;
        this.messageShower = messageShower;
        /**
         * Opções da mensagem
         */
        this.showMessageOptions = new ShowMessageOptions();
    }
    /**
     * Método para exibir uma mensagem totalmente customizada
     * @param {?} showMessageOptions
     * @return {?}
     */
    displayMessage(showMessageOptions) {
        showMessageOptions = this.messageOptionsDefaults(showMessageOptions);
        this.messageShower.displayMessage(showMessageOptions);
    }
    /**
     * Método para exibir uma mensagem internacionalizada
     * @param {?} message
     * @param {?=} messageOptions
     * @return {?}
     */
    showMessage(message, messageOptions) {
        if (!messageOptions || messageOptions === undefined || messageOptions == null) {
            messageOptions = new ShowMessageOptions();
        }
        if (!messageOptions.messageType || messageOptions.messageType === undefined || messageOptions.messageType == null) {
            messageOptions.messageType = message.messageType;
        }
        /** @type {?} */
        const msgI18n = this.i18n.text(message);
        messageOptions.messageText = (messageOptions.messageText &&
            messageOptions.messageText !== undefined && messageOptions.messageText.trim() !== '') ?
            messageOptions.messageText : msgI18n.text;
        messageOptions.messageDetails = (messageOptions.messageDetails &&
            messageOptions.messageDetails !== undefined && messageOptions.messageDetails.trim() !== '') ?
            messageOptions.messageDetails : msgI18n.detailText;
        messageOptions.messageTitle = msgI18n.titleText ? msgI18n.titleText.trim() : messageOptions.messageTitle;
        this.displayMessage(messageOptions);
    }
    /**
     * Método responsável por definir as opções padrões de uma mensagem
     * @private
     * @param {?} msgOpts
     * @return {?}
     */
    messageOptionsDefaults(msgOpts) {
        this.showMessageOptions.messageType = msgOpts.messageType;
        this.showMessageOptions.messageText = msgOpts.messageText;
        this.showMessageOptions.messageTitle = msgOpts.messageTitle;
        this.showMessageOptions.messageDetails = msgOpts.messageDetails;
        this.showMessageOptions.messageLinkActionType = msgOpts.messageLinkActionType ? msgOpts.messageLinkActionType :
            ShowMessageLinkActionType.No;
        this.showMessageOptions.messageLinkAction = msgOpts.messageLinkAction ? msgOpts.messageLinkAction : undefined;
        this.showMessageOptions.messageLinkLabel = msgOpts.messageLinkLabel ? msgOpts.messageLinkLabel : 'Clique aqui';
        this.showMessageOptions.messageConfirmAction = msgOpts.messageConfirmAction ? msgOpts.messageConfirmAction :
            undefined;
        this.showMessageOptions.messageConfirmLabel = msgOpts.messageConfirmLabel ? msgOpts.messageConfirmLabel : 'Ok';
        this.showMessageOptions.messageCancelAction = msgOpts.messageCancelAction ? msgOpts.messageCancelAction :
            undefined;
        this.showMessageOptions.messageCancelLabel = msgOpts.messageCancelLabel ? msgOpts.messageCancelLabel : 'Cancelar';
        return this.showMessageOptions;
    }
}
MessageDisplayerService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
MessageDisplayerService.ctorParameters = () => [
    { type: I18nBaseService },
    { type: AbstractShowMessage }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class DateTimeModule {
}
DateTimeModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    DateTimePipe,
                    TimePipe
                ],
                imports: [
                    CommonModule
                ],
                exports: [
                    DateTimePipe,
                    DatePipe
                ],
                providers: [
                    DatePipe,
                    DateTimePipe,
                    DefaultDateViewConverterService,
                    {
                        provide: DateViewConverter,
                        useClass: DefaultDateViewConverterService
                    },
                    TimePipe,
                    DefaultTimeViewConverterService,
                    {
                        provide: TimeViewConverter,
                        useClass: DefaultTimeViewConverterService
                    }
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Dto no padrão do backend para enumeration
 */
class EnumerationDto {
    /**
     * Método construtor que inicializa a classe
     * @param {?} id Identificação do enumerador
     * @param {?} name Nome apresentável do enumerador
     */
    constructor(id, name) {
        this.id = id;
        this.name = name;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Cria um atributo boolean alive$ que se torna falso quando o componente for destruido
 * e um observable destroy$ que completa quando o componente for destruido.
 *
 * Exemplos de uso com observables
 * \@IsDestroyed()
 * export class MyComponent implements OnInit {
 * ...
 * timer(0, 1000).pipe(tap((x) => console.log('takeWhile', x)), takeWhile(() => this['alive$'])).subscribe();
 * timer(0, 1000).pipe(tap((x) => console.log('takeUntil', x)), takeUntil(this['destroy$'])).subscribe();
 * this['destroy$'].subscribe(() => console.log('Oh no! Somenone killed me!'));
 * }
 * @return {?}
 */
function IsDestroyed() {
    return (/**
     * @template T
     * @param {?} constructor
     * @return {?}
     */
    (constructor) => {
        // extend constructor
        return class extends constructor {
            /**
             * @param {...?} args
             */
            constructor(...args) {
                super(...args);
                this.alive$ = true;
                this.destroy$ = new Subject();
                this.superDestroy = (/**
                 * @return {?}
                 */
                () => { });
                // tslint:disable-next-line:no-string-literal
                if (super['ngOnDestroy']) {
                    // tslint:disable-next-line:no-string-literal
                    this.superDestroy = super['ngOnDestroy'];
                }
            }
            /**
             * @return {?}
             */
            ngOnDestroy() {
                this.superDestroy();
                this.alive$ = false;
                this.destroy$.next();
                this.destroy$.complete();
            }
        };
    });
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Utilitários e funções genéricas
 */
class UtilsFn {
    /**
     * Retorna se o que foi passado é uma instância de array
     *
     * @param {?} prop o que se deseja verificar
     * @return {?} se é um array ou não
     */
    static isArray(prop) {
        return Object.prototype.toString.call(prop) === '[object Array]';
    }
    /**
     * Em um processamento assíncrono com promises, permite que a thread aguarde o tempo informado.
     * @param {?} ms tempo que deve aguardar, em ms
     * @return {?}
     */
    static sleep(ms) {
        /** @type {?} */
        const fn = (/**
         * @param {?} resolve
         * @return {?}
         */
        resolve => {
            setTimeout(resolve, ms);
        });
        return new Promise(fn);
    }
    /**
     * Lança uma rejeição com timeout caso a promise passada não seja resolvida a tempo
     * @template T
     * @param {?} prom A promise que fará o processamento assíncrono
     * @param {?} timeout O tempo que será aguardado, até um erro de timeout ocorrer
     * @return {?}
     */
    static timeout(prom, timeout) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            let resolveFn;
            /** @type {?} */
            let cancelReject;
            /** @type {?} */
            const timeoutPromise = new Promise((/**
             * @param {?} resolve
             * @param {?} reject
             * @return {?}
             */
            (resolve, reject) => {
                cancelReject = setTimeout(reject, timeout);
                resolveFn = resolve;
            }));
            /** @type {?} */
            const results = yield Promise.all([
                prom.then((/**
                 * @param {?} x
                 * @return {?}
                 */
                x => {
                    resolveFn(x);
                    clearTimeout(cancelReject);
                    return x;
                })),
                timeoutPromise,
            ]);
            return results.shift();
        });
    }
    /**
     * Semáforo de processamento para bloqueio de concorrência em determinada sessão
     * @param {?} fn expressão que vai ser tratada como avaliação do lock, enquanto true, continuará em bloqueio
     * @param {?=} sleepMs Quantidade, em ms, que a expressão será reavaliada em um loop infinito, até que se torne verdadeira
     * @return {?}
     */
    static lock(fn, sleepMs = 50) {
        return __awaiter(this, void 0, void 0, function* () {
            while (fn()) {
                yield UtilsFn.sleep(sleepMs);
            }
        });
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Utilitários e funções genéricas para uso em sql
 */
class SqlUtilsFn {
    /**
     * Converte os valores de um objeto para sql de inserção, separado por virgulas
     * @param {?} obj o objeto a ser convertido
     * @param {?} fields array de string contendo os nomes dos campos que se deseja converter
     * @return {?}
     */
    static objectToSqlstring(obj, fields) {
        /** @type {?} */
        let str = '';
        /** @type {?} */
        let index = 1;
        for (const prop of fields) {
            /** @type {?} */
            const con = this.serialize(obj[prop]);
            if (con !== this.DONT_SERIALIZE) {
                str += con;
                if (index < fields.length) {
                    str += ',';
                }
                index++;
            }
        }
        return str;
    }
    /**
     * Converte os valores de um objeto para array
     * @param {?} obj o objeto a ser convertido
     * @param {?} fields array de string contendo os nomes dos campos que se deseja converter
     * @return {?}
     */
    static objectToSqlValues(obj, fields) {
        /** @type {?} */
        const str = [];
        for (const prop of fields) {
            if (this.serialize(obj[prop]) !== this.DONT_SERIALIZE) {
                str.push(obj[prop]);
            }
        }
        return str;
    }
    /**
     * Converte o valor para um literal sql
     * @param {?} prop a literal sql
     * @return {?}
     */
    static serialize(prop) {
        if (UtilsFn.isArray(prop)) {
            return this.DONT_SERIALIZE;
        }
        else if (typeof prop === 'string') {
            return `'${prop}'`;
        }
        else if (typeof prop === 'number') {
            return `${prop}`;
        }
        else if (typeof prop === 'boolean') {
            return `${prop ? 1 : 0}`;
        }
        else if (typeof prop === 'undefined') {
            return `NULL`;
        }
        else if (typeof prop === 'object') {
            if (prop === null || prop === undefined) {
                return `NULL`;
            }
            else if (typeof prop.toISOString === 'function') {
                return `'${prop.toISOString()}'`;
            }
            else {
                return `'${JSON.stringify(prop)}'`;
            }
        }
    }
}
/**
 * Constante retornada quando um valor não pode ser serializado em banco sql
 */
SqlUtilsFn.DONT_SERIALIZE = '[DONT_SERIALIZE]';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Serviço fictício que representa um I18nBaseService e pode ser utilizado em módulos de testes
 */
class MockEmptyI18nBaseService extends I18nBaseService {
    /**
     * @param {?=} loader
     */
    constructor(loader) {
        super(loader);
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class LiteralLoaderMock {
    /**
     * @param {?} literal
     * @return {?}
     */
    static instance(literal) {
        /** @type {?} */
        const instance = jasmine.createSpyObj('ILiteralsLoader', ['load', 'updateOptions']);
        instance.load.and.returnValue(of(literal));
        instance.updateOptions.and.returnValue(of(literal));
        return instance;
    }
    /**
     * @private
     */
    constructor() { }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Button events to pass to `DebugElement.triggerEventHandler` for RouterLink event handler
 * @type {?}
 */
const buttonClickEvents = {
    left: { button: 0 },
    right: { button: 2 }
};
/**
 * Simulate element click. Defaults to mouse left-button click event.
 * @param {?} el
 * @param {?=} eventObj
 * @return {?}
 */
function click(el, eventObj = buttonClickEvents.left) {
    if (el instanceof HTMLElement) {
        el.click();
    }
    else {
        el.triggerEventHandler('click', eventObj);
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const configureTestSuite = (/**
 * @param {?=} configureModule
 * @return {?}
 */
(configureModule) => __awaiter(this, void 0, void 0, function* () {
    /** @type {?} */
    const testBedApi = getTestBed();
    /** @type {?} */
    const originReset = TestBed.resetTestingModule;
    beforeAll((/**
     * @return {?}
     */
    () => {
        TestBed.resetTestingModule();
        TestBed.resetTestingModule = (/**
         * @return {?}
         */
        () => TestBed);
    }));
    if (configureModule) {
        beforeAll((/**
         * @param {?} done
         * @return {?}
         */
        done => ((/**
         * @return {?}
         */
        () => __awaiter(this, void 0, void 0, function* () {
            configureModule();
            yield TestBed.compileComponents();
        })))().then(done).catch(done.fail)));
    }
    afterEach((/**
     * @return {?}
     */
    () => {
        testBedApi._activeFixtures.forEach((/**
         * @param {?} fixture
         * @return {?}
         */
        (fixture) => fixture.destroy()));
        testBedApi._instantiated = false;
    }));
    afterAll((/**
     * @return {?}
     */
    () => {
        TestBed.resetTestingModule = originReset;
        TestBed.resetTestingModule();
    }));
}));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @return {?}
 */
function spyOnLocalStorage() {
    /** @type {?} */
    let store = {};
    spyOn(localStorage, 'getItem').and.callFake((/**
     * @param {?} key
     * @return {?}
     */
    (key) => {
        return store[key] || null;
    }));
    spyOn(localStorage, 'removeItem').and.callFake((/**
     * @param {?} key
     * @return {?}
     */
    (key) => {
        delete store[key];
    }));
    spyOn(localStorage, 'setItem').and.callFake((/**
     * @param {?} key
     * @param {?} value
     * @return {?}
     */
    (key, value) => {
        return store[key] = (/** @type {?} */ (value));
    }));
    spyOn(localStorage, 'clear').and.callFake((/**
     * @return {?}
     */
    () => {
        store = {};
    }));
}

export { AUTH_SETTINGS, AUTH_TYPE, AbstractShowMessage, ApiDateTimeConstants, ApiDateTimeService, ApiFilter, ApiOrderField, ApiOrderType, ApiQueryString, AuthBaseService, AuthHandlerType, AuthInterceptorHandlerService, BaseMessage, CriticalError, CurrentUser, DateTimeModule, DateTimePipe, DateViewConverter, DefaultDateViewConverterService, DefaultResponseDto, DefaultTimeViewConverterService, EOData, EnumerationDto, FieldValue, FormUtils, FormUtilsWithService, HttpInterceptorBaseService, I18nBaseService, IsDestroyed, ListDto, ListDtoMapper, LiteralLoaderMock, LoadingDefault, MappedGetAll, MappedHttp, MappedHttpDelete, MappedHttpGet, MappedHttpGetWithQueryFilter, MappedHttpGetWithQueryString, MappedHttpPost, MappedHttpPostLocation, MappedHttpPut, MessageDisplayerService, MessageType, MessageUtils, MockEmptyI18nBaseService, NO_ERROR_HEADER_PARAM, OData, ODataAnd, ODataNot, ODataOr, QueryFilter, ShowMessageLinkActionType, ShowMessageOptions, ShowMessageType, SqlUtilsFn, TextMessage, TimePipe, TimeViewConverter, TotvsMessageType, USER_PERMISSION_SETTINGS, UrlMapper, UrlUtils, UserPermission, UserPermissionService, UtilsFn, buttonClickEvents, click, configureTestSuite, spyOnLocalStorage };
//# sourceMappingURL=totvs-log-base-foundation.js.map
