import { IQueryFilter } from './../query-filter.interface';
import { IGetAllQueryFilter } from './../get-all/get-all-query-filter.interface';
import { IGetEntity } from './verbs/get-entity.interface';
import { HttpClient } from '@angular/common/http';
import { Observable } from 'rxjs';
import { IMapper } from './../mappers/mapper.interface';
import { MappedHttp } from './mapped-http';
/**
 * Realiza a operação de Get para o enpoint informado, mapeando a entidade após recebe-la, com opção de filtros na string
 *
 * @template In Entidade recebida
 * @template Out Tipo em que a Entidade recebida será convertida
 */
export declare class MappedHttpGetWithQueryFilter<In, Out> extends MappedHttp implements IGetEntity<Out>, IGetAllQueryFilter {
    private mapper;
    private httpClient;
    private apiQuery;
    /**
     * Método responsável por definir os filtros por query que devem ser aplicados a consulta
     * @param queryFilter Classe que contém a definição dos filtros
     */
    setQueryFilter(queryFilter: IQueryFilter): void;
    /**
     * Cria uma instância de MappedHttpGetWithQueryFilter.
     * @param endpoint aceita o formato /api/entity/{0}/other/{1}
     * @param mapper o conversor para o endpoint
     * @param httpClient o cliente http
     */
    constructor(endpoint: string, mapper: IMapper<In, Out>, httpClient: HttpClient, urlMapper?: IMapper<Array<any>, string>);
    /**
     * Implementação da chamada de leitura de um recurso
     *
     * @param params parâmetros da url
     * @returns Deve ser assinado para realizar a leitura do serviço
     */
    Get(...params: Array<any>): Observable<Out>;
}
