import { IGetEntityWithQueryString } from './verbs/get-entity-with-query-string.interface';
import { HttpClient } from '@angular/common/http';
import { Observable } from 'rxjs';
import { IMapper } from './../mappers/mapper.interface';
import { MappedHttp } from './mapped-http';
/**
 * Realiza a operação de Get para o enpoint informado, mapeando a entidade antes de enviar
 *
 * @template In Entidade recebida
 * @template Out Tipo em que a Entidade recebida será convertida
 */
export declare class MappedHttpGetWithQueryString<In, Out> extends MappedHttp implements IGetEntityWithQueryString<Out> {
    private mapper;
    private httpClient;
    private _expands;
    /** Expansões para a consulta */
    readonly expands: string[];
    private _fields;
    /** Campos para a consulta */
    readonly fields: string[];
    /**
     * Creates an instance of MappedHttpGet.
     * @param endpoint aceita o formato /api/entity/{0}/other/{1}
     * @param mapper o conversor para o endpoint
     * @param httpClient o cliente http
     */
    constructor(endpoint: string, mapper: IMapper<In, Out>, httpClient: HttpClient, urlMapper?: IMapper<Array<any>, string>);
    /**
     * Método responsável por definir as expansões para a consulta
     * @param expands Lista de expansões
     */
    setExpands(...expands: string[]): void;
    /**
     * Método responsável por definir os campos para a consulta
     * @param fields Lista de campos
     */
    setFields(...fields: string[]): void;
    /**
     * Implementação da chamada de leitura de um recurso
     *
     * @param params parâmetros da url
     * @returns Deve ser assinado para realizar a leitura do serviço
     */
    Get(...params: Array<any>): Observable<Out>;
    /**
     * Retorna os parâmetros de consulta(expand e fields) em formato string
     */
    private getQueryString;
}
