import { IPostEntity } from './verbs/post-entity.interface';
import { HttpClient } from '@angular/common/http';
import { Observable } from 'rxjs';
import { IMapper } from './../mappers/mapper.interface';
import { MappedHttp } from './mapped-http';
/**
 * Realiza a operação de Post para o enpoint informado, mapeando a entidade antes de enviar
 *
 * @template In Entidade recebida
 * @template Out Tipo em que a Entidade recebida será convertida
 * @template Response Estrutura de resposta esperada pelo backend
 */
export declare class MappedHttpPost<In, Out, Response> extends MappedHttp implements IPostEntity<In, Response> {
    private mapper;
    private httpClient;
    /**
     * Creates an instance of MappedHttpPost.
     * @param endpoint aceita o formato /api/entity/{0}/other/{1}
     * @param mapper o conversor para o endpoint
     * @param httpClient o cliente http
     */
    constructor(endpoint: string, mapper: IMapper<In, Out>, httpClient: HttpClient, urlMapper?: IMapper<Array<any>, string>);
    /**
     * Implementação da chamada de criação de um recurso
     *
     * @param data dados de entrada para o serviço
     * @param params parâmetros da url
     * @returns Deve ser assinado para realizar a criação do recurso
     */
    Post(data?: In, ...params: any[]): Observable<Response>;
}
