import { II18nBaseService } from './i18n-base-service.interface';
import { ILiterals } from './literals.interface';
import { ILiteralsLoader } from './literals-loader.interface';
import { II18nUpdateOptions } from './i18n-update-options.interface';
import { Observable } from 'rxjs';
import { II18nBaseOptions } from './i18n-base-options.interface';
import { ITextMessage } from './../messaging/text-message.interface';
import { IBaseMessage } from './../messaging/base-message.interface';
import { II18nBaseLanguage } from './i18n-base-language.interface';
/**
 * Classe base de internacionalização
 * @export
 */
export declare abstract class I18nBaseService implements II18nBaseService {
    protected literalLoader: ILiteralsLoader;
    /** Opções de internacionalização */
    protected options: II18nBaseOptions;
    /** Literais */
    literals: ILiterals;
    /**
     * Construtor da classe
     */
    constructor(literalLoader: ILiteralsLoader);
    /**
     * Método responsável por setar os atributos definidos da internacionalização context, language, url
     */
    updateOptions(options: II18nUpdateOptions): Observable<ILiterals>;
    /**
     * Método responsável por setar o formato do valor
     */
    format(data: any[], text: string): string;
    /**
     * Método responsável por buscar e setar a literal do detalhe da mensagem
     */
    text(baseMsg: IBaseMessage): ITextMessage;
    /**
     * Método responsável por retornar se tem literal
     */
    hasLiteral(literal: string): boolean;
    /**
     * Método responsável por retornar as opções de linguagem
     */
    getLanguage(): string;
    /**
     * Método responsável por retornar os idiomas suportados pela aplicação
     */
    getSupportedLanguages(): Array<II18nBaseLanguage>;
}
