import { ShowMessageType } from './show-message/show-message-type.enum';
import { IBaseMessage } from './base-message.interface';
import { II18nBaseService } from '../i18n/i18n-base-service.interface';
import { ITextMessage } from './text-message.interface';
/**
 * Classe base de mensagens
 */
export declare class BaseMessage implements IBaseMessage {
    /** Tipo da mensagem que carrega */
    private _messageType;
    /** Dados para formatar a Literal */
    private _literalData;
    /**  Dados para formatar o detalhe */
    private _detailData;
    /** Literal identificador */
    private _literal;
    /** Literal identificador do detalhe */
    private _detail;
    /** Tipo da mensagem que carrega */
    readonly messageType: ShowMessageType;
    /** Builder para setar o tipo de mensagem */
    withMessageType(messageType: ShowMessageType): IBaseMessage;
    /**
     * Método que seta a literal
     */
    /**
    * Método que busca a literal
    */
    literal: string;
    /**
     * Método que seta o detalhe da literal
     */
    /**
    * Método que busca o detalhe da literal
    */
    detail: string;
    /**
     * Método construtor da classe
     */
    constructor(msg: {
        literal: string;
        detail?: string;
    });
    /**
     * Facilita a adição de dados para formatação da literal
     */
    withLiteralData(data: any[]): this;
    /**
     * Método que busca a literal
     */
    readonly literalData: any[];
    /**
     * Facilita a adição de dados para formatação do detalhe da literal
     */
    withDetailData(data: any[]): this;
    /**
     * Método que busca o detalhe da literal
     */
    readonly detailData: any[];
    /**
     * Método que busca a literal formatada
     */
    readonly literalFormattable: boolean;
    /**
     * Método que busca o detalhe da literal formatado
     */
    readonly detailFormattable: boolean;
    /**
     * Método de internacionalização de mensagem
     */
    messageI18n(i18n: II18nBaseService): ITextMessage;
    /**
     * Método que verifica se literal é válida
     */
    protected isLiteralValid(literal: string): void;
}
